From mboxrd@z Thu Jan  1 00:00:00 1970
Date: Tue, 29 Jun 2021 13:42:46 +0000
From: Klemens Nanni <klemens@posteo.de>
To: list+catgirl@causal.agency
Subject: Re: [PATCH] Perform TLS handshake after final pledge
Message-ID: <YNsjVjmK/GK2aZZU@eru>
References: <20210629122414.30838-1-klemens@posteo.de>
 <YNsU38b0gYeYxu+z@eru>
 <YNsiELop4/VFa4oI@eru>
MIME-Version: 1.0
Content-Type: text/plain; charset=us-ascii
Content-Disposition: inline
In-Reply-To: <YNsiELop4/VFa4oI@eru>

Here's the same patch with ircHandshake() after ircConnect() inside
irc.c as intended;  no other change.

-- >8 --


>From 4c3ca47c9f88623d163e8f5846a16b6c925cb7dc Mon Sep 17 00:00:00 2001
From: Klemens Nanni <klemens@posteo.de>
Date: Tue, 29 Jun 2021 15:34:03 +0200
Subject: [PATCH] Perform TLS handshake after final pledge

ircConnect() yields a connected TCP socket after which "inet dns" is
no longer needed.

Possibly having loaded private key material, it seems a tad more
comforting to speak TLS *after* dropping any network capabilities
(except for socket read/write to the IRC host, of course).

Instead of moving the final pledge into irc.c:ircConnect() and thus
complicating the code around pledge across two C modules, simply
stub out an mnemonic ircHandshake() and call that explicitly.

This restores behaviour gained with
981ebc4 "Remove explicit tls_handshake(3) from ircConnect" which
was reverted for other reasons.
---
 chat.c | 3 +++
 chat.h | 1 +
 irc.c  | 6 +++++-
 3 files changed, 9 insertions(+), 1 deletion(-)

diff --git a/chat.c b/chat.c
index ab0678a..b7562f6 100644
--- a/chat.c
+++ b/chat.c
@@ -240,6 +240,7 @@ int main(int argc, char *argv[]) {
 		if (error) err(EX_OSERR, "pledge");
 #endif
 		ircConnect(bind, host, port);
+		ircHandshake();
 		ircPrintCert();
 		ircClose();
 		return EX_OK;
@@ -322,6 +323,8 @@ int main(int argc, char *argv[]) {
 	if (error) err(EX_OSERR, "pledge");
 #endif
 
+	ircHandshake();
+
 #ifdef __FreeBSD__
 	cap_rights_t rights;
 	caph_stream_rights(&rights, CAPH_WRITE);
diff --git a/chat.h b/chat.h
index a4c7670..39d4f7f 100644
--- a/chat.h
+++ b/chat.h
@@ -234,6 +234,7 @@ void ircConfig(
 	bool insecure, const char *trust, const char *cert, const char *priv
 );
 int ircConnect(const char *bind, const char *host, const char *port);
+void ircHandshake(void);
 void ircPrintCert(void);
 void ircRecv(void);
 void ircSend(const char *ptr, size_t len);
diff --git a/irc.c b/irc.c
index c308e46..6f0d496 100644
--- a/irc.c
+++ b/irc.c
@@ -163,13 +163,17 @@ int ircConnect(const char *bindHost, const char *host, const char *port) {
 	error = tls_connect_socket(client, sock, host);
 	if (error) errx(EX_PROTOCOL, "tls_connect: %s", tls_error(client));
 
+	return sock;
+}
+
+void ircHandshake(void) {
+	int error;
 	do {
 		error = tls_handshake(client);
 	} while (error == TLS_WANT_POLLIN || error == TLS_WANT_POLLOUT);
 	if (error) errx(EX_PROTOCOL, "tls_handshake: %s", tls_error(client));
 
 	tls_config_clear_keys(config);
-	return sock;
 }
 
 void ircPrintCert(void) {
-- 
2.32.0


